#include "crewWidget.h"
#include <cassert>
#include "softProj\settings.h"

CrewWidget::CrewWidget(QWidget * parent) : NotifyWidget(parent) {
	ui_.setupUi(this);
		
	addCrewDialog_ = NULL;

	ui_.captain->setDescription(L"Bez kapitna je posdka neorganizovan, jejich vkon kles na 50%");

	ui_.doctors->setName(L"Doktoi");
	ui_.doctors->setIcon(":/pictures/images/others/doktori.png");
	ui_.doctors->setDescription(L"Doktoi zaprv poskytuj prvn pomoc pi toku, m vrazn sniuj ztrty, a zadruh li rann");

	ui_.engineers->setName(L"Ineni");
	ui_.engineers->setIcon(":/pictures/images/others/inzenyri.png");
	ui_.engineers->setDescription(L"Ineni zefektivuj prci android; nemohou je vak pln zastoupit, pouze je posiluj a zlepuj");

	ui_.officers->setName(L"Dstojnci");
	ui_.officers->setIcon(":/pictures/images/others/dustojnici.png");
	ui_.officers->setDescription(L"Dstojnci mohou nahradit libovolnho lena posdky (i kapitna), avak efektivita jejich prce na jakkoliv pozici je ni");

	ui_.passengers->setName(L"Pasai");
	ui_.passengers->setIcon(":/pictures/images/others/pasazeri.png");
	ui_.passengers->setDescription(L"Pasai sice zkladn i lodi nijak prospn nejsou, nicmn mohou za meziplanetrn i mezihvzdnou pepravu zaplatit ulrei");

	ui_.pilots->setName(L"Piloti");
	ui_.pilots->setIcon(":/pictures/images/others/piloti.png");
	ui_.pilots->setDescription(L"Piloti pomhaj potai dit lo, m zlepuj jej manvrovatelnost, co zvyuje jej bojovou schopnost. Na zkladnch nemaj efekt");

	ui_.scientists->setName(L"Vdci");
	ui_.scientists->setIcon(":/pictures/images/others/vedci.png");
	ui_.scientists->setDescription(L"Vdci pracuj na vzkumu novch technologi. Dky potaov sti spolupracuj na vzkumu ze vech zkladen a lod");

	ui_.crewMemberWeightValue->setText(QString::fromUtf16(L"Hmotnost jedn osoby"));

	bool connected = true;		
	connected = connected && connect(ui_.captain, SIGNAL(addCrewButtonClickedSignal()), this, SLOT(addCaptainSlot()));
	connected = connected && connect(ui_.captain, SIGNAL(removeCrewButtonClickedSignal()), this, SLOT(removeCaptainSlot()));
	connected = connected && connect(ui_.officers, SIGNAL(addCrewButtonClickedSignal()), this, SLOT(addOfficersSlot()));
	connected = connected && connect(ui_.officers, SIGNAL(removeCrewButtonClickedSignal()), this, SLOT(removeOfficersSlot()));
	connected = connected && connect(ui_.pilots, SIGNAL(addCrewButtonClickedSignal()), this, SLOT(addPilotsSlot()));
	connected = connected && connect(ui_.pilots, SIGNAL(removeCrewButtonClickedSignal()), this, SLOT(removePilotsSlot()));
	connected = connected && connect(ui_.scientists, SIGNAL(addCrewButtonClickedSignal()), this, SLOT(addScientistsSlot()));
	connected = connected && connect(ui_.scientists, SIGNAL(removeCrewButtonClickedSignal()), this, SLOT(removeScientistsSlot()));
	connected = connected && connect(ui_.doctors, SIGNAL(addCrewButtonClickedSignal()), this, SLOT(addDoctorsSlot()));
	connected = connected && connect(ui_.doctors, SIGNAL(removeCrewButtonClickedSignal()), this, SLOT(removeDoctorsSlot()));
	connected = connected && connect(ui_.engineers, SIGNAL(addCrewButtonClickedSignal()), this, SLOT(addEngineersSlot()));
	connected = connected && connect(ui_.engineers, SIGNAL(removeCrewButtonClickedSignal()), this, SLOT(removeEngineersSlot()));
	connected = connected && connect(ui_.passengers, SIGNAL(addCrewButtonClickedSignal()), this, SLOT(addPassengersSlot()));
	connected = connected && connect(ui_.passengers, SIGNAL(removeCrewButtonClickedSignal()), this, SLOT(removePassengersSlot()));
	assert(connected && "Signals not connected");

	memberWeightUnit_ = " kg";

	ui_.crewSection->setName(L"Posdka");
}

void CrewWidget::setState(crew_status & state) {
	ui_.captain->setStatus(state.ABLE_CREW_CAPITAN, state.WOUNDED_CREW_CAPITAN);

	ui_.doctors->setValues(state.ABLE_CREW_DOCTOR, state.WOUNDED_CREW_DOCTOR, state.TOTAL_CREW_DOCTOR);
	ui_.engineers->setValues(state.ABLE_CREW_ENGINEER, state.WOUNDED_CREW_ENGINEER, state.TOTAL_CREW_ENGINEER);
	ui_.officers->setValues(state.ABLE_CREW_OFFICER, state.WOUNDED_CREW_OFFICER, state.TOTAL_CREW_OFFICER);
	ui_.passengers->setValues(state.ABLE_CREW_PASSENGER, state.WOUNDED_CREW_PASSENGER, state.TOTAL_CREW_PASSENGER);
	ui_.pilots->setValues(state.ABLE_CREW_PILOT, state.WOUNDED_CREW_PILOT, state.TOTAL_CREW_PILOT);
	ui_.scientists->setValues(state.ABLE_CREW_SCIENTIST, state.WOUNDED_CREW_SCIENTIST, state.TOTAL_CREW_SCIENTIST);

	ui_.crewMemberWeightValue->setText(Settings::instance().formatNumber(state.ACT_PEOPLE_WEIGHT) + memberWeightUnit_);

	this->state_ = state;
}

void CrewWidget::notifyUpdate() {
}

void CrewWidget::addCaptainSlot() {
	showAddCrewDialog(CREW_TYPE_CAPTAIN, true);
}

void CrewWidget::addOfficersSlot() {
	showAddCrewDialog(CREW_TYPE_OFFICERS, true);
}

void CrewWidget::addPilotsSlot() {
	showAddCrewDialog(CREW_TYPE_PILOTS, true);
}

void CrewWidget::addEngineersSlot() {
	showAddCrewDialog(CREW_TYPE_ENGINEERS, true);
}

void CrewWidget::addScientistsSlot(){
	showAddCrewDialog(CREW_TYPE_SCIENTISTS, true);
}

void CrewWidget::addDoctorsSlot() {
	showAddCrewDialog(CREW_TYPE_DOCTORS, true);
}

void CrewWidget::addPassengersSlot() {
	showAddCrewDialog(CREW_TYPE_PASSENGERS, true);
}

void CrewWidget::removeCaptainSlot() {
	showAddCrewDialog(CREW_TYPE_CAPTAIN, false);
}

void CrewWidget::removeOfficersSlot() {
	showAddCrewDialog(CREW_TYPE_OFFICERS, false);
}

void CrewWidget::removePilotsSlot() {
	showAddCrewDialog(CREW_TYPE_PILOTS, false);
}

void CrewWidget::removeEngineersSlot() {
	showAddCrewDialog(CREW_TYPE_ENGINEERS, false);
}

void CrewWidget::removeScientistsSlot() {
	showAddCrewDialog(CREW_TYPE_SCIENTISTS, false);
}

void CrewWidget::removeDoctorsSlot() {
	showAddCrewDialog(CREW_TYPE_DOCTORS, false);
}

void CrewWidget::removePassengersSlot() {
	showAddCrewDialog(CREW_TYPE_PASSENGERS, false);
}
	
void CrewWidget::createAddCrewDialog() {
	addCrewDialog_ = new AddCrewDialog(this);
		
	bool connected = true;		
	connected = connected && connect(addCrewDialog_, SIGNAL(submitSignal()), this, SLOT(submitAddCrewDialogSlot()));
	connected = connected && connect(addCrewDialog_, SIGNAL(cancelSignal()), this, SLOT(cancelAddCrewDialogSlot()));
	assert(connected && "Signals not connected");

	addCrewDialog_->setWindowFlags(Qt::SplashScreen);
}

void CrewWidget::submitAddCrewDialogSlot() {
	if (addCrewDialog_->isAdding()) {
		emit addCrewSignal(addCrewDialog_->getAble(), addCrewDialog_->getWounded(), addCrewDialog_->getCrewType());
	}
	else {
		emit removeCrewSignal(addCrewDialog_->getAble(), addCrewDialog_->getWounded(), addCrewDialog_->getCrewType());
	}

	addCrewDialog_->close();
}

void CrewWidget::cancelAddCrewDialogSlot() {
	addCrewDialog_->close();
	emit hideDialogSignal();
}

void CrewWidget::showAddCrewDialog(crew_type crewType, bool isAdding) {
	if (addCrewDialog_ == NULL) {
		createAddCrewDialog();
	}

	emit showDialogSignal();

	addCrewDialog_->setDialog(state_, crewType, isAdding);

	addCrewDialog_->show();
	addCrewDialog_->raise();
	addCrewDialog_->activateWindow();
}