#ifndef FACTORY_LARGER_WIDGET_H
#define FACTORY_LARGER_WIDGET_H

#include <QtGui>
#include "ui_factoryLargerWidget.h"
#include "softProj\clientServer\structs.h"
#include "softProj\settings.h"

class FactoryLargerWidget : public QWidget {
	Q_OBJECT

private:
	Ui::FactoryLargerWidget ui_;

	QString consumptionUnit_;
	QString productionUnit_;

	QString availableButtonStyle_;
	QString unavailableButtonStyle_;

	QString availableDamageStyle_;
	QString unavailableDamageStyle_;
signals:
	void unmountButtonClickedSignal(bool checked);
private slots:
	void unmountButtonClickedSlot(bool checked) {
		emit unmountButtonClickedSignal(checked);
	}
public:
	FactoryLargerWidget(QWidget * parent) : QWidget(parent) {
		ui_.setupUi(this);

		availableButtonStyle_ = Settings::instance().getStyle(STYLE_AVAILABLE_DEVICE_BUTTON);
		unavailableButtonStyle_ = Settings::instance().getStyle(STYLE_UNAVAILABLE_DEVICE_BUTTON	);
		availableDamageStyle_ = Settings::instance().getStyle(STYLE_AVAILABLE_DEVICE_DAMAGE_LABEL);
		unavailableDamageStyle_ = Settings::instance().getStyle(STYLE_UNAVAILABLE_DEVICE_DAMAGE_LABEL);

		bool connected = true;
		connected = connected && connect(ui_.unmountButton, SIGNAL(clicked(bool)), this, SLOT(unmountButtonClickedSlot(bool)));
		assert(connected && "Signals not connected");
	}

	void setName(const std::wstring & name) {
		ui_.nameButton->setText(QString::fromUtf16(name.c_str()));
	}

	void setConsumptionUnit(const std::wstring & unit) {
		consumptionUnit_ = " " + QString::fromUtf16(unit.c_str());
	}

	void setConsumption(unsigned long value) {
		ui_.consumptionValue->setText(Settings::instance().formatNumber(value) + consumptionUnit_);
	}

	void setProductionUnit(const std::wstring & unit) {
		productionUnit_ = " " + QString::fromUtf16(unit.c_str());
	}

	void setProduction(unsigned long value) {
		ui_.productionValue->setText(Settings::instance().formatNumber(value) + productionUnit_);
	}

	void setDamageStatus(unsigned long long working, unsigned long long damaged) {
		ui_.damageSlider->setWorkingAndDamaged(working, damaged);
	}

	void setEffectivity(unsigned long value) {
		ui_.effectivityValue->setText(Settings::instance().formatNumber(value));
	}

	void setSpaceEffectivity(unsigned long value) {
		ui_.spaceEffectivityValue->setText(Settings::instance().formatNumber(value));
	}

	void setAvailable(bool available) {
		if (available) {
			ui_.nameButton->setStyleSheet(availableButtonStyle_);
			ui_.damageLabel->setStyleSheet(availableDamageStyle_);
			ui_.damageLabel->setText(QString::fromUtf16(L"Poet pracujcch a pokozench"));
			ui_.damageSlider->setVisible(true);
			ui_.valuesFrame->setVisible(true);
			ui_.unmountButton->setEnabled(true);
			ui_.unmountButton->setText(QString::fromUtf16(L"Odmontovat zazen"));
		}
		else {
			ui_.nameButton->setStyleSheet(unavailableButtonStyle_);
			ui_.damageLabel->setStyleSheet(unavailableDamageStyle_);
			ui_.damageLabel->setText(QString::fromUtf16(L"Zazen nen k dispozici"));
			ui_.damageSlider->setVisible(false);
			ui_.valuesFrame->setVisible(false);
			ui_.unmountButton->setEnabled(false);
			ui_.unmountButton->setText(QString::fromUtf16(L"Nen namontovno"));
		}
	}

	void setConsumptionVisible(bool visible) {
		ui_.consumptionLabel->setVisible(visible);
		ui_.consumptionValue->setVisible(visible);
	}

	void setProductionVisible(bool visible) {
		ui_.productionLabel->setVisible(visible);
		ui_.productionValue->setVisible(visible);
	}

	void setEffectivityVisible(bool visible) {
		ui_.effectivityLabel->setVisible(visible);
		ui_.effectivityValue->setVisible(visible);
	}

	void setSpaceEffectivityVisible(bool visible) {
		ui_.spaceEffectivityLabel->setVisible(visible);
		ui_.spaceEffectivityValue->setVisible(visible);
	}
};

#endif