#ifndef IMPULS_DRIVE_ENGINE_WIDGET_H
#define IMPULS_DRIVE_ENGINE_WIDGET_H

#include <QtGui>
#include "ui_impulsDriveEngineWidget.h"
#include "softProj\clientServer\structs.h"
#include "softProj\settings.h"

///Displays impuls drive engine info.
class ImpulsDriveEngineWidget : public QWidget {
	Q_OBJECT

private:
	Ui::ImpulsDriveEngineWidget ui_;

	QString consumptionUnit_;
	QString forceUnit_;

	QString availableButtonStyle_;
	QString unavailableButtonStyle_;

	QString availableDamageStyle_;
	QString unavailableDamageStyle_;
signals:
	void unmountButtonClickedSignal(bool checked);
private slots:
	void unmountButtonClickedSlot(bool checked) {
		emit unmountButtonClickedSignal(checked);
	}
public:
	ImpulsDriveEngineWidget(QWidget * parent) : QWidget(parent) {
		ui_.setupUi(this);

		ui_.damageSlider->setMinimum(0);
		ui_.damageSlider->setMaximum(100);

		ui_.damageSlider->setUnit(L"%");

		availableButtonStyle_ = Settings::instance().getStyle(STYLE_AVAILABLE_DEVICE_BUTTON);
		unavailableButtonStyle_ = Settings::instance().getStyle(STYLE_UNAVAILABLE_DEVICE_BUTTON	);
		availableDamageStyle_ = Settings::instance().getStyle(STYLE_AVAILABLE_DEVICE_DAMAGE_LABEL);
		unavailableDamageStyle_ = Settings::instance().getStyle(STYLE_UNAVAILABLE_DEVICE_DAMAGE_LABEL);

		bool connected = true;
		connected = connected && connect(ui_.unmountButton, SIGNAL(clicked(bool)), this, SLOT(unmountButtonClickedSlot(bool)));
		assert(connected && "Signals not connected");
	}


	void setName(const std::wstring & name) {
		ui_.nameButton->setText(QString::fromUtf16(name.c_str()));
	} /**< Set and show title of the  of the impuls drive engine. */

	void setDescription(const std::wstring & name) {
		ui_.descriptionLabel->setText(QString::fromUtf16(name.c_str()));
	} /**< Set and show description of the  of the impuls drive engine. */

	void setConsumptionUnit(const std::wstring & unit) {
		consumptionUnit_ = " " + QString::fromUtf16(unit.c_str());
	} /**< Set and show consumption unit of the impuls drive engine. */

	void setForceUnit(const std::wstring & unit) {
		forceUnit_ = " " + QString::fromUtf16(unit.c_str());
	} /**< Set and show force unit of the impuls drive engine. */

	void setConsumption(unsigned long value) {
		ui_.consumptionValue->setText(Settings::instance().formatNumber(value) + consumptionUnit_);
	} /**< Set and show consumption of the impuls drive engine. */

	void setForce(unsigned long value) {
		ui_.forceValue->setText(Settings::instance().formatNumber(value) + forceUnit_);
	} /**< Set and show force of the impuls drive engine. */

	void setDamageStatus(unsigned long value) {
		ui_.damageSlider->setValue(value);		
	} /**< Set and show how damaged is the impuls drive engine. */

	void setAvailable(bool available) {
		if (available) {
			ui_.nameButton->setStyleSheet(availableButtonStyle_);
			ui_.damageLabel->setStyleSheet(availableDamageStyle_);
			ui_.damageLabel->setText(QString::fromUtf16(L"Stav zazen"));
			ui_.damageSlider->setVisible(true);
			ui_.valuesFrame->setVisible(true);
			ui_.unmountButton->setEnabled(true);
			ui_.unmountButton->setText(QString::fromUtf16(L"Odmontovat"));
		}
		else {
			ui_.nameButton->setStyleSheet(unavailableButtonStyle_);
			ui_.damageLabel->setStyleSheet(unavailableDamageStyle_);
			ui_.damageLabel->setText(QString::fromUtf16(L"Zazen nen k dispozici"));
			ui_.damageSlider->setVisible(false);
			ui_.valuesFrame->setVisible(false);
			ui_.unmountButton->setEnabled(false);
			ui_.unmountButton->setText(QString::fromUtf16(L"Nenamontovno"));
		}
	}
};

#endif