#ifndef MOUNT_DIALOG_H
#define MOUNT_DIALOG_H

#include <QtGui>
#include "ui_mountDialog.h"
#include "softProj\settings.h"
#include "softProj\commonTypes.h"
#include <cassert>

///Dialog which is displayed during mounting item
class MountDialog : public QDialog
{
    Q_OBJECT

private:
	Ui::MountDialog ui_;
	std::string patternName_;
	id_t itemId_;

	 QIcon okIcon;
	 QIcon notokIcon;
	 QIcon waitIcon;

	void showIfHidden() {
		if (!isVisible()) {
			show();
			raise();
			activateWindow();
		}
	}
signals:
	void submitSignal();
	void cancelSignal();
private slots:
	void submitButtonClickedSlot(bool checked) {
		emit submitSignal();
	}

	void cancelButtonClickedSlot(bool checked) {
		emit cancelSignal();
	}
public:
	virtual void show() {
		QDialog::show();
	}
public:
	MountDialog(QWidget * parent) : QDialog(parent) {
		ui_.setupUi(this);

		bool ok = QObject::connect(ui_.submitButton, SIGNAL(clicked(bool)), this, SLOT(submitButtonClickedSlot(bool)));
		ok = ok && QObject::connect(ui_.cancelButton, SIGNAL(clicked(bool)), this, SLOT(cancelButtonClickedSlot(bool)));
		assert(ok && "Signals not connected");

        okIcon.addFile(QString::fromUtf8(":/pictures/images/trade/ok-big.png"), QSize(), QIcon::Normal, QIcon::Off);
		notokIcon.addFile(QString::fromUtf8(":/pictures/images/trade/not-ok-big.png"), QSize(), QIcon::Normal, QIcon::Off);
		waitIcon.addFile(QString::fromUtf8(":/pictures/images/trade/wait-icon.png"), QSize(), QIcon::Normal, QIcon::Off);

		ui_.countSlider->setFontColor(QColor::fromRgb(40, 40, 40));
	}

	void setTitle(const std::wstring & title) {
		ui_.titleLabel->setText(QString::fromUtf16(title.c_str()));
	}

	void setMessage(const std::wstring & message) {
		ui_.desciptionLabel->setText(QString::fromUtf16(message.c_str()));
	}

	void setDialog(const std::string & patternName, id_t itemId, unsigned long countMax) {
		ui_.resultImageButton->setVisible(false);

		ui_.countSlider->setMinimum(0);
		ui_.countSlider->setMaximum(countMax);
		ui_.countSlider->setVisible(true);
		ui_.countLabel->setVisible(true);

		this->patternName_ = patternName;
		this->itemId_ = itemId;

		setTitle(L"Montovn zazen " + Settings::instance().getGameName(patternName));
		setMessage(L"Opravdu chce zazen namontovat? Pokud ano, tak zadej poadovan mnostv.");

		ui_.cancelButton->setVisible(true);
		ui_.cancelButton->setGeometry(45, 215, ui_.cancelButton->width(), ui_.cancelButton->height());
		ui_.cancelButton->setText(QString::fromUtf16(L"Zruit, nic nemnit"));

		ui_.submitButton->setVisible(true);
		ui_.submitButton->setGeometry(205, 215, ui_.submitButton->width(), ui_.submitButton->height());
		ui_.submitButton->setText(QString::fromUtf16(L"Potvrdit a namontovat"));

		showIfHidden();
	}

	void setDialog(const std::string & patternName, id_t itemId) {
		ui_.resultImageButton->setVisible(false);

		ui_.countSlider->setVisible(false);
		ui_.countLabel->setVisible(false);

		setTitle(L"Odmontovn zazen " + Settings::instance().getGameName(patternName));
		setMessage(L"Opravdu chce zazen namontovat?");

		this->patternName_ = patternName;
		this->itemId_ = itemId;

		ui_.cancelButton->setVisible(true);
		ui_.cancelButton->setGeometry(45, 215, ui_.cancelButton->width(), ui_.cancelButton->height());
		ui_.cancelButton->setText(QString::fromUtf16(L"Zruit, nic nemnit"));

		ui_.submitButton->setVisible(true);
		ui_.submitButton->setGeometry(205, 215, ui_.submitButton->width(), ui_.submitButton->height());
		ui_.submitButton->setText(QString::fromUtf16(L"Potvrdit a namontovat"));

		showIfHidden();
	}

	void setAsDoneSuccess(const std::string & patternName, const std::string & mountingResult) {
		ui_.resultImageButton->setVisible(true);
        ui_.resultImageButton->setIcon(okIcon);
        ui_.resultImageButton->setIconSize(QSize(60, 60));		

		ui_.countSlider->setVisible(false);
		ui_.countLabel->setVisible(false);

		setTitle(L"Odmontovn zazen " + Settings::instance().getGameName(patternName));
		setMessage(L"Server potvrdil dost o namontovn pedmtu.\n" + Settings::instance().getGameName(mountingResult) );	

		ui_.cancelButton->setVisible(false);

		ui_.submitButton->setVisible(true);
		ui_.submitButton->setGeometry(120, 215, ui_.submitButton->width(), ui_.submitButton->height());
		ui_.submitButton->setText(QString::fromUtf16(L"Zavt okno"));

		showIfHidden();
	}

	void setAsDoneFail(const std::string & patternName, const std::string & mountingResult) {
		ui_.resultImageButton->setVisible(true);
        ui_.resultImageButton->setIcon(notokIcon);
        ui_.resultImageButton->setIconSize(QSize(60, 60));		

		ui_.countSlider->setVisible(false);
		ui_.countLabel->setVisible(false);

		setMessage(L"Server potvrdil dost o namontovn pedmtu.\n" + Settings::instance().getGameName(mountingResult) );	

		ui_.cancelButton->setVisible(false);

		ui_.submitButton->setVisible(true);
		ui_.submitButton->setGeometry(120, 215, ui_.submitButton->width(), ui_.submitButton->height());
		ui_.submitButton->setText(QString::fromUtf16(L"Zavt okno"));

		showIfHidden();
	}

	unsigned long getCount() {
		return ui_.countSlider->getValue();
	}

	std::string getPatternName() {
		return patternName_;
	}

	id_t getItemId() {
		return itemId_;
	}

	~MountDialog() {};
};

#endif