#ifndef NAVIGATION_DIALOG_H
#define NAVIGATION_DIALOG_H

#include <QtGui>
#include "ui_navigationDialog.h"
#include "softProj\clientServer\structs.h"
#include "softProj\settings.h"
#include <cassert>

class NavigationDialog : public QDialog
{
    Q_OBJECT
private:
	Ui::NavigationDialog ui_;
	bool canDockDown_;
signals:
	void moveStandardSignal();
	void moveSubspaceSignal();
	void cancelSignal();
	void dockDownSignal();
private slots:
	void standardButtonClickedSlot(bool checked) {
		emit moveStandardSignal();
	}

	void subspaceButtonClickedSlot(bool checked) {
		emit moveSubspaceSignal();
	}

	void dockButtonClickedSlot(bool checked) {
		emit dockDownSignal();
	}

	void cancelButtonClickedSlot(bool checked) {
		emit cancelSignal();
	}
public:
	NavigationDialog(QWidget * parent) : QDialog(parent) {
		ui_.setupUi(this);

		bool ok = true;
		ok = ok && QObject::connect(ui_.standardButton, SIGNAL(clicked(bool)), this, SLOT(standardButtonClickedSlot(bool)));
		ok = ok && QObject::connect(ui_.subspaceButton, SIGNAL(clicked(bool)), this, SLOT(subspaceButtonClickedSlot(bool)));
		ok = ok && QObject::connect(ui_.dockButton, SIGNAL(clicked(bool)), this, SLOT(dockButtonClickedSlot(bool)));
		ok = ok && QObject::connect(ui_.cancelButton, SIGNAL(clicked(bool)), this, SLOT(cancelButtonClickedSlot(bool)));
		ok = ok && QObject::connect(ui_.planetCancelButton, SIGNAL(clicked(bool)), this, SLOT(cancelButtonClickedSlot(bool)));

		ui_.hoursEdit->setValidator(new QRegExpValidator(QRegExp("[0-9]*"), ui_.hoursEdit));
		ui_.minutesEdit->setValidator(new QRegExpValidator(QRegExp("[0-9]*"), ui_.minutesEdit));
		ui_.secondsEdit->setValidator(new QRegExpValidator(QRegExp("[0-9]*"), ui_.secondsEdit));

		assert(ok && "Signals not connected");
	}

	~NavigationDialog() {};

	int getHours() {
		return ui_.hoursEdit->text().toInt();
	}

	int getMinutes() {
		return ui_.minutesEdit->text().toInt();
	}

	int getSeconds() {
		return ui_.secondsEdit->text().toInt();
	}

	void setDialog(bool showWarp) {
		if (showWarp) {			
			ui_.desciptionLabel->setText(QString::fromUtf16(L"Na zvolen msto lze lett podprostorovm nebo impulsnm pohonem. Pokud chce lett impulsnm pohonem, zadej jet dobu zehu."));
		}
		else {			
			ui_.desciptionLabel->setText(QString::fromUtf16(L"Na zvolen msto lze lett pouze impulsnm pohonem. Zadej poadovanou dobu zehu."));
		}

		ui_.standardLabel->setText(QString::fromUtf16(L"Lo me lett impulsnm pohonem"));

		if (showWarp) {
			ui_.subspaceLabel->setProperty("enabled", true);
			ui_.subspaceLabel->setStyleSheet(ui_.subspaceLabel->styleSheet());
			ui_.subspaceLabel->setText(QString::fromUtf16(L"Lo me lett podprostorovm pohonem"));
			ui_.subspaceButton->setVisible(true);
		}
		else {
			ui_.subspaceLabel->setProperty("enabled", false);
			ui_.subspaceLabel->setStyleSheet(ui_.subspaceLabel->styleSheet());
			ui_.subspaceLabel->setText(QString::fromUtf16(L"Lo neme lett podprostorovm pohonem"));
			ui_.subspaceButton->setVisible(false);
		}

		resize(404, 349);
		ui_.imageFrame->resize(400, 345);
		ui_.mainFrame->resize(400, 345);
		ui_.planetFrame->setVisible(false);

		ui_.dockLabel->setVisible(false);
		ui_.dockReasonLabel->setVisible(false);
		ui_.dockButton->setVisible(false);

		ui_.cancelButton->setVisible(true);
	}

	void setDialog(DockingResult canDockDown) {
		ui_.desciptionLabel->setText(QString::fromUtf16(L"Na zvolen msto lze lett pouze impulsnm pohonem. Zadej poadovanou dobu zehu."));

		ui_.standardLabel->setText(QString::fromUtf16(L"Lo me lett impulsnm pohonem"));

		ui_.subspaceLabel->setProperty("enabled", false);
		ui_.subspaceLabel->setStyleSheet(ui_.subspaceLabel->styleSheet());
		ui_.subspaceLabel->setText(QString::fromUtf16(L"Lo neme lett podprostorovm pohonem"));
		ui_.subspaceButton->setVisible(false);

		ui_.planetFrame->setVisible(false);

		if (canDockDown == DR_OK) {
			ui_.dockLabel->setProperty("enabled", true);
			ui_.dockLabel->setStyleSheet(ui_.dockLabel->styleSheet());
			ui_.dockLabel->setText(QString::fromUtf16(L"Lo me pistt na zkladn"));
			ui_.dockReasonLabel->setText(QString::fromUtf16(L"Ve v podku"));
			ui_.dockButton->setVisible(true);
		}
		else {
			ui_.dockLabel->setProperty("enabled", false);
			ui_.dockLabel->setStyleSheet(ui_.dockLabel->styleSheet());
			ui_.dockLabel->setText(QString::fromUtf16(L"Lo neme pistt na zkladn"));

			std::wstring reason;
			switch (canDockDown) {
			case DR_UNCONTROLLABLE_TARGET:
				reason = L"Clov zkladna je nekontrolovateln, lze ji obsadit, ale ne na ni pistt";
				break;
			case DR_NOT_YOUR_DOCK:
				reason = L"Clovou zkladnu vlastn jin hr, nelze tam proto pistt";
				break;
			case DR_OUT_OF_RANGE:
				reason = L"Clov zkladna nen v dosahu";
				break;
			case DR_IMPULS_DRIVE_FAIL:
				reason = L"Lo nem motory i tlumie nebo je nco z toho rozbit";
				break;
			case DR_NOT_ENOUGH_DEUTERIUM:
				reason = L"lo nem dost paliva na pistn";
			case DR_NO_FREE_HANGAR:
				reason = L"Clov zkladna nem dn voln hangr";
				break;
			default:
				reason = L"Nelze pistt";
			}

			ui_.dockButton->setVisible(false);
			ui_.dockReasonLabel->setText(QString::fromUtf16(reason.c_str()));
		}

		resize(404, 504);
		ui_.imageFrame->resize(400, 500);
		ui_.mainFrame->resize(400, 500);
		ui_.planetFrame->setVisible(false);

		ui_.cancelButton->setVisible(true);
	}

	void setDialog(const NavigationPlanetDetails & planet) {
		ui_.desciptionLabel->setText(QString::fromUtf16(L"Na zvolen msto lze lett podprostorovm nebo impulsnm pohonem. Pokud chce lett impulsnm pohonem, zadej jet dobu zehu."));

		ui_.standardLabel->setText(QString::fromUtf16(L"Lo me lett impulsnm pohonem"));

		ui_.subspaceLabel->setProperty("enabled", true);
		ui_.subspaceLabel->setStyleSheet(ui_.subspaceLabel->styleSheet());
		ui_.subspaceLabel->setText(QString::fromUtf16(L"Lo me lett podprostorovm pohonem"));
		ui_.subspaceButton->setVisible(true);

		resize(804, 504);
		ui_.imageFrame->resize(800, 500);
		ui_.mainFrame->resize(400, 500);
		ui_.planetFrame->setVisible(true);

		ui_.dockLabel->setVisible(false);
		ui_.dockReasonLabel->setVisible(false);
		ui_.dockButton->setVisible(false);

		ui_.planetName->setText(QString::fromUtf8(planet.name.c_str()));
		ui_.heavyWaterModifier->setText(Settings::instance().formatNumber(planet.heavyWaterModifier * 100) + " %");
		ui_.crystalsModifier->setText(Settings::instance().formatNumber(planet.crystalsModifier * 100) + " %");
		ui_.ilmeniteModifier->setText(Settings::instance().formatNumber(planet.ilmeniteModifier * 100) + " %");

		ui_.dayTemperature->setText(Settings::instance().formatNumber(planet.dayTemperature) + QString::fromUtf16(L" C"));
		ui_.nightTemperature->setText(Settings::instance().formatNumber(planet.nightTemperature) + QString::fromUtf16(L" C"));

		ui_.dayLength->setText(Settings::instance().formatNumber(planet.dayPeriod) + " s");
		ui_.yearLength->setText(Settings::instance().formatNumber(planet.yearPeriod) + " s");

		ui_.sea->setText(QString::number(planet.sea));

		if (planet.type == GAS_TITAN) {
			ui_.planetType->setText(QString::fromUtf16(L"Plynov obr"));
		}
		else if (planet.type == TERESTRIAL) {
			ui_.planetType->setText(QString::fromUtf16(L"Terestick planeta"));
		}
		else {
			ui_.planetType->setText(QString::fromUtf16(L"Neznm"));
		}

		ui_.cancelButton->setVisible(false);

	}
};

#endif