#ifndef NAVIGATION_COMPUTER_WIDGET_H
#define NAVIGATION_COMPUTER_WIDGET_H

#include <QtGui>
#include <cassert>
#include "ui_navigationComputerWidget.h"
#include "softProj\clientServer\structs.h"
#include "softProj\settings.h"

///Manages ship navigation issues.
class NavigationComputerWidget : public QWidget {
	Q_OBJECT

private:
	Ui::NavigationComputerWidget ui_;

	QString speedUnit_;

	QString availableButtonStyle_;
	QString unavailableButtonStyle_;

	QString availableDamageStyle_;
	QString unavailableDamageStyle_;
signals:
	void unmountButtonClickedSignal(bool checked);
private slots:
	void unmountButtonClickedSlot(bool checked) {
		emit unmountButtonClickedSignal(checked);
	}
public:
	NavigationComputerWidget(QWidget * parent) : QWidget(parent) {
		ui_.setupUi(this);

		ui_.damageSlider->setMinimum(0);
		ui_.damageSlider->setMaximum(100);

		ui_.damageSlider->setUnit(L"%");

		availableButtonStyle_ = Settings::instance().getStyle(STYLE_AVAILABLE_DEVICE_BUTTON);
		unavailableButtonStyle_ = Settings::instance().getStyle(STYLE_UNAVAILABLE_DEVICE_BUTTON	);
		availableDamageStyle_ = Settings::instance().getStyle(STYLE_AVAILABLE_DEVICE_DAMAGE_LABEL);
		unavailableDamageStyle_ = Settings::instance().getStyle(STYLE_UNAVAILABLE_DEVICE_DAMAGE_LABEL);

		bool connected = true;
		connected = connected && connect(ui_.unmountButton, SIGNAL(clicked(bool)), this, SLOT(unmountButtonClickedSlot(bool)));
		assert(connected && "Signals not connected");
	}

	void setName(const std::wstring & name) {
		ui_.nameButton->setText(QString::fromUtf16(name.c_str()));
	} /**< Sets title name of the navigation computer. */

	void setDescription(const std::wstring & name) {
		ui_.descriptionLabel->setText(QString::fromUtf16(name.c_str()));
	} /**< Sets description text of the navigation computer. */

	void setSpeedUnit(const std::wstring & unit) {
		speedUnit_ = " " + QString::fromUtf16(unit.c_str());
	} /**< Sets speed unit of the navigation computer. */

	void setSpeed(unsigned long value) {
		ui_.speedValue->setText(Settings::instance().formatNumber(value) + speedUnit_);
	} /**< Sets current ship speed in navigation computer. */

	void setDamageStatus(unsigned long value) {
		ui_.damageSlider->setValue(value);		
	} /**< Sets damage state of the navigation computer. */

	void setAvailable(bool available) {
		if (available) {
			ui_.nameButton->setStyleSheet(availableButtonStyle_);
			ui_.damageLabel->setStyleSheet(availableDamageStyle_);
			ui_.damageLabel->setText(QString::fromUtf16(L"Stav zazen"));
			ui_.damageSlider->setVisible(true);
			ui_.valuesFrame->setVisible(true);
			ui_.unmountButton->setEnabled(true);
			ui_.unmountButton->setText(QString::fromUtf16(L"Odmontovat"));
		}
		else {
			ui_.nameButton->setStyleSheet(unavailableButtonStyle_);
			ui_.damageLabel->setStyleSheet(unavailableDamageStyle_);
			ui_.damageLabel->setText(QString::fromUtf16(L"Zazen nen k dispozici"));
			ui_.damageSlider->setVisible(false);
			ui_.valuesFrame->setVisible(false);
			ui_.unmountButton->setEnabled(false);
			ui_.unmountButton->setText(QString::fromUtf16(L"Nenamontovno"));
		}
	}
};

#endif