#include "settings.h"
#include "softProj\clientServer\threads.h"
#include "softProj\clientServer\structs.h"

threads::mutex * Settings::lock_ = new threads::mutex();
Settings * Settings::instance_ = NULL;

Settings::Settings() 
{	
	createStyles();
	createGameNames();
	createGameIcons();
}

Settings::~Settings()
{
}

Settings & Settings::instance() 
{
	if (instance_ == NULL)
	{
		lock_->lock();

		if (instance_ == NULL) 
		{
			instance_ = new Settings();
		}

		lock_->unlock();
	}

	return *instance_;
}

void Settings::playSoundSlot() 
{
	QSound::play("../sounds/button.wav");
}

void Settings::connectSound(QPushButton * button) 
{
	//QObject::connect(
	//	button,  SIGNAL(clicked()),
	//	this, SLOT(playSoundSlot())
	//	);
}

void Settings::connectStyle(QWidget * widget, const std::string & key) {
	widget->setStyleSheet(getStyle(key));
}

void Settings::createStyles() {

	emptyStyle_ = "";

	/* Style for name button of device that is available, like in factories widget */

	QString availableButtonStyle = 
		"QPushButton {"
			"background-color: rgb(48, 66, 35, 230);"
			"padding: 5px;"
			"color: #ffffff;"
			"font-weight: bold;"
			"border: none;"
			"text-align: center;"
		"}";

	styles_.insert(std::make_pair(STYLE_AVAILABLE_DEVICE_BUTTON, availableButtonStyle));

	QString availableSmallButtonStyle = 
		"QPushButton {"
			"background-color: rgb(48, 66, 35, 230);"
			"color: #ffffff;"
			"font-weight: normal;"
			"border: none;"
			"text-align: center;"
			"font-size: 10px;"
		"}";

	styles_.insert(std::make_pair(STYLE_AVAILABLE_DEVICE_SMALL_BUTTON, availableSmallButtonStyle));

	/* Style for name button of device that is unavailable, like in factories widget */

	QString unavailableButtonStyle = 
		"QPushButton {"
			"background-color: rgb(50, 50, 50);"
			"padding: 5px;"
			"color: #ffffff;"
			"border: none;"
			"font-weight: bold;"
		"}";

	styles_.insert(std::make_pair(STYLE_UNAVAILABLE_DEVICE_BUTTON, unavailableButtonStyle));

	QString unavailableSmallButtonStyle = 
		"QPushButton {"
			"background-color: rgb(50, 50, 50);"
			"color: #ffffff;"
			"border: none;"
			"font-weight: normal;"
			"font-size: 10px;"
		"}";

	styles_.insert(std::make_pair(STYLE_UNAVAILABLE_DEVICE_SMALL_BUTTON, unavailableSmallButtonStyle));

	/* Style for damage label of device that is available, like in factories widget */

	QString availableDamageStyle = 
		"QLabel  {"
			"background-color: rgb(35, 35, 35, 140);"
			"color: rgb(75, 120, 175);"
			"padding: 5px;"
			"font-weight: bold;"
		"}";

	styles_.insert(std::make_pair(STYLE_AVAILABLE_DEVICE_DAMAGE_LABEL, availableDamageStyle));

	/* Style for damage label of device that is unavailable, like in factories widget */

	QString unavailableDamageStyle = 
		"QLabel  {"
			"background-color: rgb(35, 35, 35, 140);"
			"color: rgb(170, 45, 44);"
			"padding: 5px;"
			"font-weight: bold;"
		"}";

	styles_.insert(std::make_pair(STYLE_UNAVAILABLE_DEVICE_DAMAGE_LABEL, unavailableDamageStyle));
}

void Settings::createGameNames() {

	emptyGameName_ = L"";

	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_ANDROIDS, L"Androidi"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_CLOAKING_DEVICE, L"Maskovac zazen"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_DEFLECTOR_SHIELDS, L"Deflektor tt"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_DEUTERIUM_DECOMPOSER, L"Rozklada deuteria"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_DRONE_MISSILES, L"Dronn stela"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_DRONE_MISSILE_LAUNCHER, L"Torpdomet"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_ENERGETIC_ACCUMULATOR, L"Energetick akumultor"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_ENERGETIC_CELL, L"Energetick lnek"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_HANGAR, L"Hangr"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_HIGH_RESOLUTION_SENSORS, L"Citliv senzory"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_ILMENITE_FURNACE, L"Ilmenitov pec"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_IMPULS_JET_ENGINE, L"Engine impulznho motoru"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_INERTIAL_DAMPERS, L"Inerciln tlumie"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_LASER_GUN, L"Laserov dlo"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_LIFE_SUPPORT, L"Podpora ivota"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_LONG_RANGE_SENSORS, L"Dlkov senzory"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_MAGNETICAL_COLLECTORS_OF_PLASMA, L"Magnetick kolektory plazmy"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_MAGNETICAL_FILTER, L"Magnetick filtr"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_MAGNETICAL_GRID, L"Magnetick mka"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_MAINFRAME, L"Palubn pota"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_MECHANICAL_DAMPERS, L"Mechanick tlumie"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_NAVIGATION_COMPUTER, L"Navigan pota"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_PLASMA_FILTER, L"Filtr plazmy"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_PROBE, L"Przkumn zazen"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_QUARK_GLUON_CORE, L"Quark-gluonov jdro"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_SCIENTIFIC_LAB, L"Vdeck laborato"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_SUBMARINE_MINING_MACHINE, L"Podmosk tebn zazen"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_SUBMERSIBLE_FILTRATION_TANK, L"Ponorn filtran ndr"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_SUBSPACE_BEACON, L"Podprostorov majk"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_SUBSPACE_TRANSMITTER, L"Podprostorov vysla"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_TRANSPORT_BEAM_GENERATOR, L"Genertor transportnho paprsku"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_TRANSPORT_CONTAINER, L"Transportn kontejner"));	
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_UNDERGROUND_DRILLIG_MACHINE, L"Podzemn tebn stroje"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_WEAPON_ENERGY_CIRCUIT, L"Energetick zbraov okruh"));	
	gameNames_.insert(std::make_pair(PROTOCOL_HANGARS_PATTERN_ID_ZIRCONIUM_REACTOR, L"Zirkoniov reaktor"));	

	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_DONE, L"Ve v podku"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_FIRST_FORTRESS_CAPACITY_LIMIT, L"Pekroen kapacitn limit v prvn pevnosti"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_FIRST_FORTRESS_ENERGETIC_CELLS_LACK, L"Nedostatek energetickch lnk v prvn pevnosti"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_FIRST_FORTRESS_MAGNETICAL_GRIDS_LACK, L"Nedostatek magnetickch mek v prvn pevnosti"));	
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_FIRST_FORTRESS_NOT_STORING_ITEMS, L"Prvn pevnost nem nkter pedmty z nabdky"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_FIRST_FORTRESS_TRANSPORT_CONTAINERS_LACK, L"Nedostatek transportnch kontejner v prvn pevnosti"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_FIRST_FORTRESS_WEIGHT_LIMIT, L"Pekroen hmotnost limit v prvn pevnosti"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_SECOND_FORTRESS_CAPACITY_LIMIT, L"Pekroen kapacitn limit v druh pevnosti"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_SECOND_FORTRESS_ENERGETIC_CELLS_LACK, L"Nedostatek energetickch lnk v druh pevnosti"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_SECOND_FORTRESS_MAGNETICAL_GRIDS_LACK, L"Nedostatek magnetickch mek v druh pevnosti"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_SECOND_FORTRESS_NOT_STORING_ITEMS, L"Druh pevnost nem nkter pedmty z nabdky"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_SECOND_FORTRESS_TRANSPORT_CONTAINERS_LACK, L"Nedostatek transportnch kontejner v druh pevnosti"));
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_SECOND_FORTRESS_WEIGHT_LIMIT, L"Pekroen hmotnost limit v druh pevnosti"));	
	gameNames_.insert(std::make_pair(PROTOCOL_TRADE_RESULT_NOT_IN_RANGE, L"Pevnosti nejsou v dosahu pro obchodovn"));	

	gameNames_.insert(std::make_pair(PROTOCOL_MOUNTING_RESULT_DONE, L"Ve v podku"));
	gameNames_.insert(std::make_pair(PROTOCOL_MOUNTING_RESULT_SLOT_OCCUPIED, L"Pedmt nelze namontovat, protoe na jeho mst u je jin pedmt, kter je teba nejprve demontovat."));
	gameNames_.insert(std::make_pair(PROTOCOL_MOUNTING_SLOT_EMPTY, L"Pedmt nelze demontovat, protoe tam dn nen."));
	gameNames_.insert(std::make_pair(PROTOCOL_MOUNTING_RESULT_NOT_ENOUGH_ANDROIDS, L"Pedmt nelze montovat ani odmontovat, protoe nem dostatek android."));
	gameNames_.insert(std::make_pair(PROTOCOL_MOUNTING_RESULT_NOT_ENOUGH_ENERGY, L"Pedmt nelze montovat ani odmontovat, protoe nem dostatek energie"));
	gameNames_.insert(std::make_pair(PROTOCOL_MOUNTING_RESULT_NOT_ENOUGH_STORAGE_SPACE, L"Pedmt nelze odmontovat, protoe je pln sklad."));
	gameNames_.insert(std::make_pair(PROTOCOL_MOUNTING_RESULT_UNKNOWN_FAIL, L"Neznm pina, zejm chyba na serveru. "));			
	gameNames_.insert(std::make_pair(PROTOCOL_MOUNTING_RESULT_HANGAR_NOT_EMPTY, L"Hangr nelze namontovat ani odmontovat, protoe v sob m njak vyrbn pedmt nebo lo"));	
	gameNames_.insert(std::make_pair(PROTOCOL_MOUNTING_RESULT_BAD_FORTRESS_TYPE, L"Pedmt nelze namontovat ani odmontovat, protoe v danm typu pevnosti nen. "));	

	gameNames_.insert(std::make_pair(REQUEST_CHANGE_DEVICE_ENERGY_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_CHANGE_DEVICE_ENERGY_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));	

	gameNames_.insert(std::make_pair(REQUEST_DEVICE_ENERGY_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_DEVICE_ENERGY_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));	

	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_WEAPONS_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_WEAPONS_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));

	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_CREW_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_CREW_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));

	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_ACCOMMODATIONS_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_ACCOMMODATIONS_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));

	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_OTHER_EQUIPMENT_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_OTHER_EQUIPMENT_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));

	gameNames_.insert(std::make_pair(REQUEST_BASE_HANGARS_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_BASE_HANGARS_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_BASE_HANGARS_INFORMATION_ANSWER_FORTRESSNOTBASE, L"Zadan pevnost nen zkladna."));

	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_WAREHOUSES_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_WAREHOUSES_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));

	gameNames_.insert(std::make_pair(REQUEST_SHIP_ANDROIDS_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_SHIP_ANDROIDS_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_SHIP_ANDROIDS_INFORMATION_ANSWER_FORTRESSNOTSHIP, L"Zadan pevnost nen lo."));

	gameNames_.insert(std::make_pair(REQUEST_BASE_ANDROIDS_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_BASE_ANDROIDS_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_BASE_ANDROIDS_INFORMATION_ANSWER_FORTRESSNOTBASE, L"Zadan pevnost nen zkladna."));

	gameNames_.insert(std::make_pair(REQUEST_CHANGE_SHIP_ANDROIDS_STATE_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_CHANGE_SHIP_ANDROIDS_STATE_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_CHANGE_SHIP_ANDROIDS_STATE_ANSWER_FORTRESSNOTSHIP, L"Zadan pevnost nen lo."));

	gameNames_.insert(std::make_pair(REQUEST_CHANGE_BASE_ANDROIDS_STATE_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_CHANGE_BASE_ANDROIDS_STATE_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_CHANGE_BASE_ANDROIDS_STATE_ANSWER_FORTRESSNOTBASE, L"Zadan pevnost nen zkladna."));

	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_ADD_ACCOMMODATIONS_ANSWER_FORTRESSNE, L"Zadan pevnost nen zkladna."));
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_ADD_ACCOMMODATIONS_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));	

	gameNames_.insert(std::make_pair(REQUEST_SHIP_JOURNEY_ANSWER_SHIPNE, L"Zadan lo neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_SHIP_JOURNEY_ANSWER_SHIPNO, L"Zadan lo nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_SHIP_JOURNEY_ANSWER_BADSPOBJ, L"Zadan vesmrn objekt neexistuje."));

	gameNames_.insert(std::make_pair(REQUEST_SHIPSINAREA_ANSWER_NOTEXIST, L"Zadan lo neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_SHIPSINAREA_ANSWER_NOTOWN, L"Zadan lo nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_SHIPSINAREA_ANSWER_WARP, L"Lo je ve warpu, nelze splnit poadavek."));
	gameNames_.insert(std::make_pair(REQUEST_SHIPSINAREA_ANSWER_IE, L"Intern chyba serveru."));

	gameNames_.insert(std::make_pair(REQUEST_PLANETS_BAD_ANSWER_NOT_EXISTS, L"Zadan oblasti neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_PLANETS_BAD_ANSWER_NOT_STAR, L"Zadan vesmrn objekt nen hvzda."));
	gameNames_.insert(std::make_pair(REQUEST_PLANETS_BAD_ANSWER_IE, L"Intern chyba serveru."));
	
	gameNames_.insert(std::make_pair(REQUEST_NAVIGATION_ANSWER_NOT_EXISTS, L"Zadan lo neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_NAVIGATION_ANSWER_NOT_OWNED, L"Zadan lod nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_NAVIGATION_ANSWER_NOT_CONTROLLABLE, L"Zadan lod nen kontrolovateln."));	
	gameNames_.insert(std::make_pair(REQUEST_NAVIGATION_ANSWER_IN_WARP, L"Lo je ve warpu, nelze splnit poadavek."));
	gameNames_.insert(std::make_pair(REQUEST_NAVIGATION_ANSWER_NOT_IN_WARP, L"Lo nen ve warpu, nelze splnit poadavek."));
	gameNames_.insert(std::make_pair(REQUEST_NAVIGATION_ANSWER_IN_BASE, L"Lo pistla na zkladn."));
	gameNames_.insert(std::make_pair(REQUEST_NAVIGATION_ANSWER_BAD_TARGET1, L"Clov lo nen na zkladn."));
	gameNames_.insert(std::make_pair(REQUEST_NAVIGATION_ANSWER_BAD_TARGET2, L"Clov objekt navigace neexistuje."));

	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_QUARRY_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_QUARRY_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_QUARRY_CHANGE_STATE_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_QUARRY_CHANGE_STATE_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_FORTRESS_QUARRY_CHANGE_STATE_ANSWER_BAD_TYPE, L"Zadan surovina neexistuje."));

	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_ITEM_ANSWER_FAIL_FORBIDDEN_DIRECTION, L"Pedmt se nesm pesunout do dan mnoiny."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_ITEM_ANSWER_FAIL_SOURCE_FORTRESS_NOT_EXISTS, L"Zdrojov pevnost neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_ITEM_ANSWER_FAIL_TARGET_FORTRESS_NOT_PARTNER, L"Clov pevnost s tebou neobchoduje."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_ITEM_ANSWER_FAIL_TARGET_REALM_NOT_PARTNER, L"Clov hr s tebou neobchoduje."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_ITEM_ANSWER_FAIL_SOURCE_FORTRESS_NOT_PARTNER, L"Zdrojov pevnost neobchoduje."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_ITEM_ANSWER_FAIL_YOU_ARE_NOT_TRADER, L"Nejsi ve stavu obchodovn."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_ITEM_ANSWER_FAIL_TARGET_FORTRESS_NOT_EXISTS, L"Clov pevnost neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_ITEM_ANSWER_FAIL_UNKNOWN_ITEM, L"Zadan pedmt neexistuje."));

	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_RESOURCE_ANSWER_FAIL_FORBIDDEN_DIRECTION, L"Surovina se nesm pesunout do dan mnoiny."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_RESOURCE_ANSWER_FAIL_SOURCE_FORTRESS_NOT_EXISTS, L"Zdrojov pevnost neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_RESOURCE_ANSWER_FAIL_TARGET_FORTRESS_NOT_PARTNER, L"Clov pevnost s tebou neobchoduje."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_RESOURCE_ANSWER_FAIL_TARGET_REALM_NOT_PARTNER, L"Clov hr s tebou neobchoduje."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_RESOURCE_ANSWER_FAIL_SOURCE_FORTRESS_NOT_PARTNER, L"Zdrojov pevnost neobchoduje."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_RESOURCE_ANSWER_FAIL_YOU_ARE_NOT_TRADER, L"Nejsi ve stavu obchodovn."));
	gameNames_.insert(std::make_pair(REQUEST_MOVE_TRADE_RESOURCE_ANSWER_FAIL_TARGET_FORTRESS_NOT_EXISTS, L"Clov pevnost neexistuje."));

	gameNames_.insert(std::make_pair(REQUEST_BASE_HANGARS_INFORMATION_ANSWER_FORTRESSNE, L"Zadan pevnost neexistuje."));	
	gameNames_.insert(std::make_pair(REQUEST_BASE_HANGARS_INFORMATION_ANSWER_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_BASE_HANGARS_INFORMATION_ANSWER_FORTRESSNOTBASE, L"Zadan pevnost nen zkladna."));
	gameNames_.insert(std::make_pair(REQUEST_BASE_HANGARS_INFORMATION_ANSWER_HANGAR_NOT_EXISTS, L"Zadan hangr neexistuje."));	

	gameNames_.insert(std::make_pair(REQUEST_LOGIN_ANSWER_ALREADY_AUTHORIZED, L"Zadan hr u je na serveru pihlen."));	
	gameNames_.insert(std::make_pair(REQUEST_LOGIN_ANSWER_BAD_PASSWORD, L"Zadan heslo neodpovd."));	
	gameNames_.insert(std::make_pair(REQUEST_LOGIN_ANSWER_BAD_LOGIN, L"Zadan uivatelsk jmno neodpovd."));	

	gameNames_.insert(std::make_pair(REQUEST_TRADE_PARTNER_FORTRESSES_ANSWER_FORTRESSNE, L"Clov pevnost neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_TRADE_PARTNER_FORTRESSES_ANSWER_FORTRESSNO, L"Clov pevnost nen hrova."));

	gameNames_.insert(std::make_pair(REQUEST_SEND_TRADE_INVITATION_ANSWER_FAIL_FORTRESSNO, L"Zadan pevnost nen hrova."));

	gameNames_.insert(std::make_pair(PROTOCOL_ERROR_MESSAGE_FORTRESS_NOT_CONTROLLABLE, L"Zadan pevnost nen kontrolovateln."));

	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_ACCUMULATOR, L"Akumultor"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_CHARGING, L"Nabjen"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_MAX_SPEED_TOC, L"Maximln rychlost"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_CONSUMPTION, L"Spoteba"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_ENGINE_FORCE, L"Sla motoru"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_MAX_ACCELERATION, L"Maximln akcelerace"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_FORCIBILITY, L"Tah"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_PRECISION, L"Pesnost"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_RANGE, L"Dosah"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_TOTAL_WINDING_SPEED, L"Celkov rychlost tby"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_LEVEL, L"rove"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_POINTS_PER_SECOND, L"Body za sekundu"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_WORKING_CONSUMPTION, L"Pracovn spoteba"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_CAPACITY, L"Kapacita"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_TOTAL_FILTERING_SPEED, L"Celkov rychlost filtrace"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_CONSUMPTION_FOR_ONE_PERSON, L"Spoteba na osobu"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_MESSAGE_COST, L"Cena za zprvu"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_BIO_NODES, L"Bio-uzly"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_PREDICTION, L"Predikce"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_TIME_REUSABILITY, L"Znovupouitelnost asu"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_HEIGHT, L"Vka"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_LENGTH, L"Dlka"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_WIDTH, L"ka"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_TOTAL_PRODUCTION, L"Celkov produkce"));	
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_METAMORPHOSIS_EFFECTIVITY, L"Efektivita promny"));		
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_DAMAGE, L"Pokozen"));
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_LEAK, L"Samovoln nik"));
	gameNames_.insert(std::make_pair(PROTOCOL_PARAM_NAME_SOLIDITY, L"Pevnost"));

	gameNames_.insert(std::make_pair(REQUEST_RENAME_FORTRESS_ANSWER_FAIL_FORTRESSNE, L"Zadan pevnost neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_RENAME_FORTRESS_ANSWER_FAIL_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_RENAME_FORTRESS_ANSWER_FAIL_EMPTY_NAME, L"Zadan jmno pevnosti je przdn."));

	gameNames_.insert(std::make_pair(PROTOCOL_HANGAR_ICR_OK, L"Ve v podku"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGAR_ICR_INVALID_PARAMS, L"Byly zadny neplatn parametry"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGAR_ICR_FULL_HANGAR, L"Hangr u je pln"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGAR_ICR_NOT_ENOUGH_ULREI, L"Nem dostatek ulrei"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGAR_ICR_NOT_ENOUGH_TITANIUM, L"Nem dostatek titanu"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGAR_ICR_NOT_ENOUGH_DEUTERIUM, L"Nem dostatek deuteria"));
	gameNames_.insert(std::make_pair(PROTOCOL_HANGAR_ICR_NOT_ENOUGH_HEAVY_WATER, L"Nem dostatek tk vody"));

	gameNames_.insert(std::make_pair(REQUEST_HANGAR_CANCEL_ITEM_FORTRESSNE, L"Zadan pevnost neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_HANGAR_CANCEL_ITEM_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_HANGAR_CANCEL_ITEM_FORTRESSNOTBASE, L"Zadan pevnost nen pevnost."));
	gameNames_.insert(std::make_pair(REQUEST_HANGAR_CANCEL_ITEM_HANGARNE, L"Zadan hangr neexistuje."));

	gameNames_.insert(std::make_pair(REQUEST_HANGAR_BUILD_ITEM_FORTRESSNE, L"Zadan pevnost neexistuje."));
	gameNames_.insert(std::make_pair(REQUEST_HANGAR_BUILD_ITEM_FORTRESSNO, L"Zadan pevnost nen hrova."));
	gameNames_.insert(std::make_pair(REQUEST_HANGAR_BUILD_ITEM_FORTRESSNOTBASE, L"Zadan pevnost nen pevnost."));
	gameNames_.insert(std::make_pair(REQUEST_HANGAR_BUILD_ITEM_HANGARNE, L"Zadan hangr neexistuje."));	
}

void Settings::createGameIcons() {
	emptyIcon_ = QIcon();

	QIcon accommodations;
	accommodations.addFile(QString::fromUtf8(":/pictures/images/details/ubykace.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_ACCOMMODATIONS, accommodations));	

	QIcon wait;
	wait.addFile(QString::fromUtf8(":/pictures/images/trade/wait-icon.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_WAIT, wait));

	QIcon invitation;
	invitation.addFile(QString::fromUtf8(":/pictures/images/trade/email-icon.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_INVITATION, invitation));

	QIcon success;
	success.addFile(QString::fromUtf8(":/pictures/images/trade/ok-big.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_SUCCESS, success));

	QIcon failure;
	failure.addFile(QString::fromUtf8(":/pictures/images/trade/not-ok-big.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_FAILURE, failure));

	QIcon base;
	base.addFile(QString::fromUtf8(":/pictures/images/details/modelZakladna.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_BASE, base));

	QIcon conquer;
	conquer.addFile(QString::fromUtf8(":/pictures/images/details/zbrane.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_CONQUER, conquer));

	QIcon deviceDistribution;
	deviceDistribution.addFile(QString::fromUtf8(":/pictures/images/others/energyCircuit.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_DEVICE_DITRIBUTION, deviceDistribution));	

	QIcon weapons;
	weapons.addFile(QString::fromUtf8(":/pictures/images/details/zbrane.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_WEAPONS, weapons));

	QIcon crew;
	crew.addFile(QString::fromUtf8(":/pictures/images/details/posadka.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_CREW, crew));
	
	QIcon equipment;
	equipment.addFile(QString::fromUtf8(":/pictures/images/details/vybaveni.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_EQUIPMENT, equipment));

	QIcon factories;
	factories.addFile(QString::fromUtf8(":/pictures/images/details/tovarny.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_FACTORIES, factories));

	QIcon warehouses;
	warehouses.addFile(QString::fromUtf8(":/pictures/images/details/sklady.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_WAREHOUSES, warehouses));

	QIcon androids;
	androids.addFile(QString::fromUtf8(":/pictures/images/details/androidi.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_ANDROIDS, androids));

	QIcon impulsDrive;
	impulsDrive.addFile(QString::fromUtf8(":/pictures/images/details/zakladniPohon.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_IMPULS_DRIVE, impulsDrive));

	QIcon subspaceDrive;
	subspaceDrive.addFile(QString::fromUtf8(":/pictures/images/details/podprostorovyPohon.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_SUBSPACE_DRIVE, subspaceDrive));

	QIcon ship;
	ship.addFile(QString::fromUtf8(":/pictures/images/others/shuttle-icon.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_SHIP, ship));

	QIcon quarrying;
	quarrying.addFile(QString::fromUtf8(":/pictures/images/details/mining.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_QUARRYING, quarrying));

	QIcon trade;
	trade.addFile(QString::fromUtf8(":/pictures/images/main/obchod.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_TRADE, trade));

	QIcon hangars;
	hangars.addFile(QString::fromUtf8(":/pictures/images/others/hangary.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_HANGARS, hangars));	

	QIcon shields;
	shields.addFile(QString::fromUtf8(":/pictures/images/details/stity.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_SHIELDS, shields));	

	QIcon centralInformation;
	centralInformation.addFile(QString::fromUtf8(":/pictures/images/details/centralInfo.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_CENTRAL_INFORMATION, centralInformation));	

	QIcon mainframe;
	mainframe.addFile(QString::fromUtf8(":/pictures/images/details/pocitac.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_MAINFRAME, mainframe));

	QIcon cloaking;
	cloaking.addFile(QString::fromUtf8(":/pictures/images/details/maskovani.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_CLOAKING, cloaking));

	QIcon sensors;
	sensors.addFile(QString::fromUtf8(":/pictures/images/details/senzory.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_SENSORS, cloaking));

	QIcon probe;
	probe.addFile(QString::fromUtf8(":/pictures/images/details/pruzkum.png"), QSize(), QIcon::Normal, QIcon::Off);
	gameIcons_.insert(std::make_pair(GAME_ICON_PROBE, cloaking));
}


const QString & Settings::getStyle(const std::string & key) {
	std::map<std::string, QString>::const_iterator it = styles_.find(key);

	if (it == styles_.end()) {
		return emptyStyle_;
	}
	
	return it->second;
}

const std::wstring & Settings::getGameName(const std::string & key) {
	std::map<std::string, std::wstring>::const_iterator it = gameNames_.find(key);

	if (it == gameNames_.end()) {
		return emptyGameName_;
	}
	
	return it->second;
}

const QIcon & Settings::getGameIcon(const std::string & key) {
	std::map<std::string, QIcon>::const_iterator it = gameIcons_.find(key);

	if (it == gameIcons_.end()) {
		return emptyIcon_;
	}
	
	return it->second;
}

QString Settings::getGameNameQString(const std::string & key) {
	return QString::fromUtf16(getGameName(key).c_str());
}

QString Settings::formatNumber(const unsigned long long & number) {
    QLocale::setDefault(QLocale::English);
    QString formatted = QString("%L1").arg(number);
	formatted.replace(",", " ");
	return formatted;
}

QString Settings::formatNumber(const unsigned long & number) {
    QLocale::setDefault(QLocale::English);
    QString formatted = QString("%L1").arg(number);
	formatted.replace(",", " ");
	return formatted;
}

QString Settings::formatNumber(const long long & number) {
    QLocale::setDefault(QLocale::English);
    QString formatted = QString("%L1").arg(number);
	formatted.replace(",", " ");
	return formatted;
}

QString Settings::formatNumber(const long & number) {
    QLocale::setDefault(QLocale::English);
    QString formatted = QString("%L1").arg(number);
	formatted.replace(",", " ");
	return formatted;
}

QString Settings::formatNumber(const unsigned int & number) {
    QLocale::setDefault(QLocale::English);
    QString formatted = QString("%L1").arg(number);
	formatted.replace(",", " ");
	return formatted;
}

QString Settings::formatNumber(const int & number) {
    QLocale::setDefault(QLocale::English);
    QString formatted = QString("%L1").arg(number);
	formatted.replace(",", " ");
	return formatted;
}

QString Settings::formatNumber(const double & number) {
    QLocale::setDefault(QLocale::English);
    QString formatted = QString("%L1").arg(number);
	formatted.replace(",", " ");
	return formatted;
}