
#include "threads.h"

using namespace threads;

#ifdef _DEBUG
#ifndef DEBUG_NEW
#define DEBUG_NEW new(_NORMAL_BLOCK, __FILE__, __LINE__)
#define new DEBUG_NEW
#endif
#endif

// ------------------------------------------------------------------------------------------------

struct threadData 
{
	runnable* r;
	thread* t;
};

// ------------------------------------------------------------------------------------------------

static DWORD WINAPI threadRunner(LPVOID data) 
{
	threadData* d = (threadData*)data;
	(*d->r)(d->t);
	// d bylo naalokovano v konstruktoru threadu
	delete d;
	return 0;
}

// ------------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------------

thread :: thread(runnable* runObj, bool suspended) 
{
	// zinicializuj data threadu
	threadData *d = new threadData;
	d->r = runObj;
	d->t = this;

	if ((threadHandle_ = CreateThread(0, 0, threadRunner, d, suspended ? CREATE_SUSPENDED : 0, 0)) == NULL) 
	{
		throw std :: exception("Cannot create thread!");
	}
}

// ------------------------------------------------------------------------------------------------

void thread :: start(void)
{
	if (ResumeThread(threadHandle_) == (DWORD) -1)
	{
		throw std :: exception("Cannot start thread!");
	}
}

// ------------------------------------------------------------------------------------------------

void thread :: join(void) 
{
	// cekej na ukonceni threadu
	if (WaitForSingleObject(threadHandle_, INFINITE) == WAIT_FAILED)
	{
		throw std :: exception("Waiting for thread termination failed");
	}

	// uzavri handle
	if (CloseHandle(threadHandle_) == 0) 
	{
		throw std :: exception("Closing thread failed");
	}

	// nastav handle na NULL, z bezpecnostnich duvodu, aby nevisel v aplikaci
	// neplatny handler
	threadHandle_ = NULL;
}

// ------------------------------------------------------------------------------------------------

bool thread :: join(DWORD miliseconds) {
	// musime osetrit ruzne pripady cekani na objekt
	switch (WaitForSingleObject(threadHandle_, miliseconds)) 
	{
	case WAIT_TIMEOUT:
		// vyprsel cas na joinovani, vrat, ze se nejoinovalo
		return false;

	case WAIT_OBJECT_0:
		// uzavri handle
		if (CloseHandle(threadHandle_) == 0) 
		{
			throw std :: exception("Closing thread failed");
		}

		// nastav handle na NULL, z bezpecnostnich duvodu, aby nevisel v aplikaci
		// neplatny handler
		threadHandle_ = NULL;

		// vrat, ze se joinovani podarilo
		return true;

	default:
		throw std :: exception("Waiting for thread termination failed");
	}
}

// ------------------------------------------------------------------------------------------------

#pragma message("Please rather do not use functions resume and suspend")

// ------------------------------------------------------------------------------------------------

void thread :: resume(void)
{
	if (ResumeThread(threadHandle_) == (DWORD)-1) 
	{
		throw std :: exception("Resume thread function failed!");
	}
}

// ------------------------------------------------------------------------------------------------

void thread :: suspend(void)
{
	if (SuspendThread(threadHandle_) == (DWORD)-1)
	{
		throw std :: exception("Suspend thread function failed!");
	}
}

// ------------------------------------------------------------------------------------------------

bool thread :: isAlive(void)
{
	DWORD status;
	if (GetExitCodeThread(threadHandle_, &status) == 0) 
	{
		throw std :: exception("GetExitCodeThread failed!");
	} 
	else
	{
		return status == STILL_ACTIVE;
	}
}

// ------------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------------

void threads :: sleepCurrentThread(DWORD miliseconds)
{
	Sleep(miliseconds);
}

// ================================================================================================

mutex :: mutex()
{
	if ((mutexHandle_ = CreateMutex(NULL, FALSE, NULL)) == NULL)
	{
		throw std :: exception("Mutex creation failed!");
	}
}

// ------------------------------------------------------------------------------------------------

mutex :: ~mutex()
{
	// uvolni handle
	if (CloseHandle(mutexHandle_) == 0) 
	{
		throw std :: exception("Closing thread failed");
	}

	mutexHandle_ = 0;
}

// ------------------------------------------------------------------------------------------------

void mutex :: lock(void)
{
	lock(INFINITE);		
}

// ------------------------------------------------------------------------------------------------

bool mutex :: lock(DWORD miliseconds)
{
	// musime osetrit ruzne pripady cekani na mutex
	switch (WaitForSingleObject(mutexHandle_, miliseconds)) 
	{
	case WAIT_TIMEOUT:
		// vyprsel cas na zamceni mutexu
		return false;

	case WAIT_OBJECT_0:
		// mutex se uzamknul
		return true;

	default:
		throw std :: exception("Waiting for mutex acquiring failed!");
	}
}

// ------------------------------------------------------------------------------------------------

void mutex :: unlock(void)
{
	if (!ReleaseMutex(mutexHandle_))
	{
		throw std :: exception("Releasing mutex failed!");
	}
}

// ================================================================================================

semaphore :: semaphore(int initcnt, int maxcnt)
{
	if ((semaphoreHandle_ = CreateSemaphore(NULL, initcnt, maxcnt, NULL)) == NULL)
	{
		throw std :: exception("Semaphore creation failed!");
	}
}

// ------------------------------------------------------------------------------------------------

semaphore :: ~semaphore()
{
	// uvolni handle
	if (CloseHandle(semaphoreHandle_) == 0) 
	{
		throw std :: exception("Closing thread failed");
	}

	semaphoreHandle_ = 0;
}

// ------------------------------------------------------------------------------------------------

void semaphore :: down(void)
{
	down(INFINITE);		
}

// ------------------------------------------------------------------------------------------------

bool semaphore :: down(DWORD miliseconds)
{
	// musime osetrit ruzne pripady cekani na semaphore
	switch (WaitForSingleObject(semaphoreHandle_, miliseconds)) 
	{
	case WAIT_TIMEOUT:
		// vyprsel cas na zamceni semaphoreu
		return false;

	case WAIT_OBJECT_0:
		// semaphore se uzamknul
		return true;

	default:
		throw std :: exception("Waiting for semaphore down failed!");
	}
}

// ------------------------------------------------------------------------------------------------

void semaphore :: up(void)
{
	if (!ReleaseSemaphore(semaphoreHandle_, 1, NULL))
	{
		throw std :: exception("Releasing semaphore failed!");
	}
}

// ================================================================================================
